#include <windows.h>

char AppName[] = "LEFTTEXT";

// [[ The code excerpt starts here]]

void PaintLeftJustifiedTitle(HWND hWnd, BOOL active)
{
    HDC hDC;
    RECT windowRect, fillRect;
    WORD frameWidth, frameHeight;
    WORD bitmapWidth, bitmapHeight;
	WORD textLen;
    COLORREF captionColor, captionTextColor;
    HFONT oldFont;
    WORD stringHeight, stringStartY;
    char buffer[128];
    
    hDC = GetWindowDC(hWnd);    // Get a DC for the entire window
    if ( !hDC )
        return;

    //
    // Get the dimensions of the Window frames and the buttons
    // at the top of the window.  Also get the colors of the
    // caption area, and the text color.  Here, we're assuming
    // a resizeable window with min and max buttons.  The colors
    // are for the active window.  Different colors are needed
    // for the inactive case.
    //
    frameWidth = GetSystemMetrics(SM_CXFRAME);
    frameHeight = GetSystemMetrics(SM_CYFRAME);
    bitmapWidth = GetSystemMetrics(SM_CXSIZE);
    bitmapHeight = GetSystemMetrics(SM_CYSIZE);
    captionColor =
        GetSysColor(active ? COLOR_ACTIVECAPTION:COLOR_INACTIVECAPTION);
    captionTextColor = 
        GetSysColor(active ? COLOR_CAPTIONTEXT:COLOR_INACTIVECAPTIONTEXT);
    
    GetWindowRect(hWnd, &windowRect);

    //
    // Calculate the coordinates of the caption window.
    //
    fillRect.left = frameWidth + bitmapWidth + 1;
    fillRect.right = (windowRect.right-windowRect.left)
                 - (frameWidth + 2*(bitmapWidth+1));
    fillRect.top = frameHeight;
    fillRect.bottom = frameHeight + bitmapHeight;

    //
    // Get the title bar text, and null terminate (if necessary)
    //
    textLen = GetWindowText(hWnd, buffer, sizeof(buffer)-1);
    buffer[ sizeof(buffer)-1 ] = 0;
    stringHeight = HIWORD( GetTextExtent(hDC, buffer, textLen) );

    //
    // Figure out how far down the text should be so that it's
    // centered in the caption area.
    //
    if ( stringHeight <= bitmapHeight)
        stringStartY = fillRect.top + (bitmapHeight - stringHeight)/2;
    else
        stringStartY = fillRect.top;

    //
    // Select the appropriate text & background colors, the background
	// mode, and the system font into the DC, then blast the string out.
	// We'll let ExtTextOut() fill in the caption rectangle, thereby
	// erasing what was put there originally by DefWindowProc().
    //
    SetTextColor(hDC, captionTextColor);
	SetBkColor(hDC, captionColor);
    SetBkMode(hDC, OPAQUE);
    oldFont = SelectObject(hDC, GetStockObject(SYSTEM_FONT));
    ExtTextOut(hDC, fillRect.left, stringStartY, ETO_CLIPPED | ETO_OPAQUE,
               &fillRect, buffer, lstrlen(buffer), 0);
    SelectObject(hDC, oldFont);
        
    ReleaseDC(hWnd, hDC);
}

LRESULT CALLBACK _export MainWndProc(HWND hWnd, unsigned iMessage,
                                     WORD wParam, LONG lParam )
{
    LRESULT retvalue;
    BOOL fActive;       // Are we painting the active window?
    
    switch( iMessage )
    {
        case WM_NCPAINT :
        case WM_NCACTIVATE:
            //
            // Let DefWindowProc() do the hard work, and then we'll
            // redo the part we care about.
            //
            retvalue = DefWindowProc(hWnd, iMessage, wParam, lParam );
        
            if ( iMessage == WM_NCACTIVATE )
                fActive = (BOOL)wParam;
            else
                fActive = (BOOL) (hWnd == GetActiveWindow());
                    
            PaintLeftJustifiedTitle(hWnd, fActive);
            return retvalue;
        case WM_DESTROY :
            PostQuitMessage(0);
            break;
   }
   
   return DefWindowProc(hWnd, iMessage, wParam, lParam );
}

// [[ The code excerpt ends here]]

int GlobalWinInit( HANDLE hInstance )
{
    WNDCLASS wndClass;

    wndClass.style = CS_HREDRAW | CS_VREDRAW;
    (FARPROC)wndClass.lpfnWndProc = (FARPROC)MainWndProc;
    wndClass.cbClsExtra = 0;
    wndClass.cbWndExtra = 0;
    wndClass.hInstance = hInstance;
    wndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
    wndClass.hbrBackground = GetStockObject(WHITE_BRUSH);
    wndClass.lpszMenuName = NULL;
    wndClass.lpszClassName = AppName;

    return RegisterClass( &wndClass );  // Return 0 if failed, TRUE otherwise
}

HWND InstanceWinInit( HANDLE hInstance )
{
    return CreateWindow(
                AppName,       // Class name
                AppName,
                WS_OVERLAPPEDWINDOW,    // Window style
                CW_USEDEFAULT,          // Initial X position
                CW_USEDEFAULT,          // Initial Y position
                CW_USEDEFAULT,          // Initial X size
                CW_USEDEFAULT,          // Initial Y size
                NULL,                   // Parent window handle
                NULL,                   // Window menu handle
                hInstance,              // progran instance handle
                NULL );                 // create parameters
}

int PASCAL WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow )
{
    HWND hWnd;
    MSG msg;

    if ( hPrevInstance == 0 )       // If the first instance,
    {                               // do a global init
        if ( GlobalWinInit(hInstance) == 0)
            return FALSE;
    }

    hWnd = InstanceWinInit( hInstance );

    ShowWindow(hWnd,nCmdShow);  // Show the window
    UpdateWindow(hWnd);         // Force a redraw

    while ( GetMessage( &msg, NULL, 0, 0 ) )
    {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    return msg.wParam;
}
